%Reads in REE data from an .xlsx file
%reads file that is two rows long and any number of columns wide
% first column is sample name
% first row is headers with element names
% second row is sample name followed by PARTITION COEFFICIENT of each element


clear all
close all

ree_order = cellstr(['Lu';'Yb';'Tm';'Er';'Ho';'Dy';'Tb';'Gd';'Eu']);
ionic_radius = [0.977 0.985 0.994 1.004 1.015 1.027 1.04 1.053 1.066]; 
D_values = ones(size(ree_order)); %initialize vector

[filename] = uigetfile('*.xlsx','Select REE data file');
[num,txt,raw] = xlsread(filename);
headers = txt(1,:);  %reads first line, which is headers
headers(1)=[];       %deletes first value, which is samples string

%the loop below reads through the file and finds the elements of interest
%it puts the values for these elements into a new vector, "ree_values"
for i = 1:length(ree_order);
    for j = 1:length(headers);
    tf=strcmpi(ree_order(i),headers(j));
    if tf==1
        D_values(i)=num(j);
    end
    end
end

fprintf('What is the temp (K) of zircon formation? (enter 1000 if unknown)\n')
temp = input('Temp?:');

%------------------------------------------------------
%Strain Lattice Calculation
%Uses equation 2 from Blundy and Wood (1994), Nature

NA = 6.0221409e+23; %Avagadros Number (1/mol)
R = 8.3144598;      %Gas constant (J/(K*mol)), J=(kg*(m^2))/(s^2)
Ro = 0.9; %ionic radius of optimal site for REE in zircon, (nm)
%Ro was determined empirically


start = round(D_values(1),-2); % rounds D(Lu) to nearest 100
poss_Do = start:50:10000; %values of Do to be tested
poss_Em = 300:25:800; %values of Youngs modulus to be tested

for i = 1:length(poss_Do); 
    for j = 1:length(poss_Em); 
        for k = 1:length(D_values);
            Di(k) = poss_Do(i)*exp((-4*pi*NA*(poss_Em(j)/1e9))/(R*temp)*(((Ro/2)...
                *((Ro-ionic_radius(k))^2))-((1/3)*((Ro-ionic_radius(k))^3)))*1e-12);
        end
            perc_err(i,j) = sum(abs(D_values'-Di)./Di);
    end
end

%finds minimum percent error value
[ind_min_Do,ind_min_Em]=find(perc_err==min(min(perc_err)));
fit_Do =  poss_Do(ind_min_Do);
fit_Em = poss_Em(ind_min_Em);


%----------------------------------------------
% Plotting results

%calculating best fit Di curve
all_radii = 0.8:0.01:1.3;
Di_plot = ones(1,length(all_radii));  %initialize
for i=1:length(all_radii)
    Di_plot(i) = fit_Do*exp((-4*pi*NA*(fit_Em/1e9))/(R*temp)*(((Ro/2)...
                *((Ro-all_radii(i))^2))-((1/3)*((Ro-all_radii(i))^3)))*1e-12);
end



figure(1)
    semilogy(ionic_radius,D_values,'ob',all_radii,Di_plot,'-r');  
    ylabel('Partition Coefficient')            
    xlabel('Ionic Radius (nm)')   
    box on;grid on; 
    tt1=title(txt(2,1));
    set(tt1,'fontsize',12)   
    ylim([0.001 10000])
    xlim([0.8 1.2])
    
str_Do = ['Do= ',num2str(fit_Do)];
str_Em = ['Em= ',num2str(fit_Em)];
    
    text(1.1,fit_Do,str_Do);
    text(1.1,fit_Do/10,str_Em);
    
